*:*****************************************************************************
*:
*: Procedure file: C:\FTS\FTS.PRG
*:
*:         System: Fox Text Search
*:         Author: Mike Lascuola
*:      Copyright (c) 1994, Mike Lascuola
*:  Last modified: 12/20/94     15:35
*:
*:  Procs & Fncts: FTS()
*:               : SETENV
*:               : TEXTSEARCH
*:               : CLEANUP
*:               : USAGE()
*:               : PARSECMDLN
*:               : GETWITHIN()
*:               : BUILDCMD()
*:
*:      Documented 12/20/94 at 16:05               FoxDoc  version 2.10f
*:*****************************************************************************
*************************************************************************
*
* FTS.PRG
* Fox Text Search Utility
*
* 04/10/94  19:20:51
*
*
* DESCRIPTION:
*    This function allows for complex searches of text files.  It
*    provides functionality I have not been able to find in GREP,
*    Norton's Text Search and File Find, and similar programs.
*    It is easier to use, however, than pattern-matching utilities
*    like AWK.
*
*
* USAGE:
*    FTS [FileSpec] [Text] [#OR#] [#AND#] [#NOT#] [/WITHIN 1 LINE]
*       [/WITHIN 1 WORD] [/S] [/CS] [/LOGLogfile]
*
*
* PARAMETERS:
*   #  Variable   Type  Description
*   -  --------   ----  -----------
*   1  lcFileSpec  C    The filespec of files to search.
*                       Example:  C:\UTIL\*.*
*
*   2  lcSrchText  C    The text for which to search.  The search text
*                       may include the keywords #AND#, #OR# or #NOT#.
*                       Example:  JOHN #AND# SMITH
*
*   3  lcWithin    C    Flag indicating search scope for keywords
*                       #AND# and #OR#.
*                       Example:  JOHN #AND# SMITH WITHIN 1 LINE
*
*   4  llSubDir    C    Flag indicating whether or not to search
*                       subdirectories also.  Defaults to .F..
*                       Example:  /S
*
*   5  llCaseSens  C    Flag indicating case sensitivity.  Default is
*                       non-case sensitive.
*                       Example:  /CS
*
*   6  llLog       C    Flag indicating redirect output to a file.
*                       If no file specified, redirect to screen.
*                       Example:  /LOG
*                       Example:  /LOGtemp.txt
*
* RETURN VALUE:
*   None.
*
*************************************************************************
FUNCTION FTS
PARAMETERS lcParam1, lcParam2, lcParam3, lcParam4, lcParam5, lcParam6

* Check usage.
lnParam = PARAMETERS()
IF NOT EMPTY(GETENV("FTS1"))
   lnParam = 1
ENDIF

IF lnParam = 0 OR lnParam > 6
   RETURN USAGE()
ENDIF

* See if running from .BAT file
glRuntime = "Support Library"$VERSION(1)

DO SetEnv   && Set the environment

PRIVATE lcFileSpec, lcSrchText, lcWithin, llSubDir, llCaseSens, llLog
PRIVATE lcLogFile

* Initialize parameter defaults (don't know until parse cmd line).
llLog      = .F.            && Interactive mode
llCaseSens = .F.            && Not case sensitive
llSubDir   = .F.            && Look only in current dir
lcWithin   = ""             && No WITHIN text
lcSrchText = ""             && Text to search for
lcFileSpec = ""             && Files to search
lcLogFile  = ""             && File to redirect output if llLog

PRIVATE lnCount    && FOR Counter
FOR lnCount = 1 TO lnParam
   lcParam = 'lcParam' + (STR(lnCount, 1))
   DO ParseCmdLn WITH &lcParam
ENDFOR


WAIT WINDOW "Searching " + FULLPATH(lcFileSpec) ;
   + "..." NOWAIT

PRIVATE lcFile   && String to hold file names
lcFile = ""

gcCommand = ""

llContinue = .T.

lcFile = SYS(2000, lcFileSpec)

IF NOT EMPTY(lcFile)
   =TextSearch(lcFile)
ENDIF

lcFile = SYS(2000, lcFileSpec, 1)

DIMENSION laFiles[500]

FOR lnCount = 1 TO 500
   laFiles[lnCount] = SYS(2000, lcFileSpec, 1)
ENDFOR

lnCount = 1

DO WHILE NOT EMPTY(laFiles[lnCount]) AND llContinue
   =TextSearch(laFiles[lnCount])
   lnCount = lnCount + 1
ENDDO

WAIT CLEAR
RETURN








*!*****************************************************************************
*!
*!      Procedure: SETENV
*!
*!      Called by: FTS()              (function  in FTS.PRG)
*!
*!*****************************************************************************
PROCEDURE SetEnv


SET TALK OFF
SET CONSOLE OFF
SET CONFIRM OFF
SET BELL OFF
SET LIBRARY TO \FOXPRO25\FPATH.PLB
ON ESCAPE DO EscHandler

* Set colors if in Runtime mode
IF glRuntime
   SET COLOR TO W/N,W+/B                  && White on Black
   CLEAR
ENDIF

RETURN


*!*****************************************************************************
*!
*!      Procedure: TEXTSEARCH
*!
*!      Called by: FTS()              (function  in FTS.PRG)
*!
*!          Calls: JUSTPATH()         (function  in FPATH.PLB)
*!               : BUILDCMD()         (function  in FTS.PRG)
*!               : CLEANUP            (procedure in FTS.PRG)
*!
*!    Other Files: LCFILE
*!
*!*****************************************************************************
PROCEDURE TextSearch
PARAMETERS lcFile

* Array to hold line of text and two previous and subsequent lines.
PRIVATE laText
DIMENSION laText[5]

PRIVATE lcString1, lcString2  && Used for #NOT#

lcString1 = ""
lcString2 = ""
gcCommand = ""

lnLine  = 0     && Line number
llFound = .F.   && .T. if text is found

* Add path to lcFile
IF NOT EMPTY(lcFile) AND NOT EMPTY(JustPath(lcFileSpec))
   lcFile = JustPath(lcFileSpec) + "\" + lcFile
ENDIF

lhHandle = FOPEN(lcFile)
IF lhHandle < 0
   WAIT WINDOW "Unable to open " + lcFile + "..." TIMEOUT 4
ELSE
   STORE FSEEK(lhHandle, 0) TO ifp_top && Move pointer to BOF
ENDIF

IF FERROR() = 0
   
   DO WHILE NOT FEOF(lhHandle)
      IF TYPE("lcLastLn1") = "C"
         lcLastLn2 = lcLastLn1
      ENDIF
      IF TYPE("lcString") = "C"
         lcLastLn1 = lcString
      ENDIF
      lcString = FGETS(lhHandle, 80)
      lnLine = lnLine + 1
      
      * Build command based on presence of #AND#, #OR#, etc.
      lcCommand = BuildCmd(lcSrchText)
      
      &lcCommand
      
      WAIT WINDOW "Searching " + FULLPATH(lcFile) + "..." NOWAIT
      
      IF llFound
         @ 0, 0 TO 12 ,80 CLEAR
         
         @ 1, 0 SAY lcSrchText + " Found in " + FULLPATH(lcFile) + ;
            ", line " + ALLTRIM(STR(lnLine)) + "."
         
         @ 3, 0 SAY lcLastLn2
         @ 4, 0 SAY lcLastLn1
         @ 5, 0 SAY lcString
         
         IF NOT llLog
            WAIT CLEAR
            @ 9, 0 SAY "Continue (Y/N)?" GET llContinue PICTURE "Y"
            READ
         ENDIF
         
         IF NOT llContinue
            DO CleanUp
         ENDIF
         WAIT WINDOW "Searching " + FULLPATH(lcFileSpec) ;
            + "..." NOWAIT
      ENDIF   && lcSrchText$lcString)
   ENDDO  &&  WHILE NOT FEOF()
   
   = FCLOSE(lhHandle)
   
ENDIF   && FERROR() = 0

RETURN


*!*****************************************************************************
*!
*!      Procedure: CLEANUP
*!
*!      Called by: TEXTSEARCH         (procedure in FTS.PRG)
*!
*!*****************************************************************************
PROCEDURE CleanUp
CLOSE ALL
RETURN TO MASTER



*!*****************************************************************************
*!
*!       Function: USAGE()
*!
*!      Called by: FTS()              (function  in FTS.PRG)
*!
*!*****************************************************************************
FUNCTION USAGE
CLEAR
? "FTS"
? ""
? "USAGE:"
? ""
? "FTS [FileSpec] [Text] [#OR#] [#AND#] [WITHIN 1 LINE]"
? "   [WITHIN 1 WORD] [/S] [/CS] [/LOGLogfile]"
?
WAIT
RETURN TO MASTER



*********************************************************************
*
* PROCEDURE: ParseCmdLn
*
* Populates global vars according to cmd line parameters or DOS
* Env vars.
*
*********************************************************************
*!*****************************************************************************
*!
*!      Procedure: PARSECMDLN
*!
*!      Called by: FTS()              (function  in FTS.PRG)
*!
*!          Calls: GETWITHIN()        (function  in FTS.PRG)
*!
*!*****************************************************************************
PROCEDURE ParseCmdLn
PARAMETERS lcParam

PRIVATE lcLogParam  && 6th parameter from DOS command line

IF NOT EMPTY(lcParam)
   lcParam = ALLTRIM(lcParam)
   
   DO CASE
   CASE LEFT(UPPER(lcParam), 4) == "/LOG"
      llLog = .T.
      
   CASE LEFT(UPPER(lcParam), 3) == "/CS"
      llCaseSens = .T.
      
   CASE LEFT(UPPER(lcParam), 2) == "/S"
      llSubDir = .T.
      
   CASE LEFT(UPPER(lcParam), 7) == "/WITHIN"
      lcWithin = GetWithin(lcParam)
      
   CASE EMPTY(lcFileSpec)
      lcFileSpec = lcParam
      
   CASE EMPTY(lcSrchText)
      lcSrchText = lcParam
      
   ENDCASE
ELSE
   
   * Running from DOS command line
   lcFileSpec = GETENV("FTS1")  && The filespec of files to search.
   lcSrchText = GETENV("FTS2")  && The text for which to search.
   
   * Flag indicating search scope for keywords #AND# and #OR#.
   lcWithin = GETENV("FTS3")
   
   * Flag indicating search subdirectories
   llSubDir = UPPER(GETENV("FTS4")) == "/S"
   
   * Flag indicating case sensitivity.  Default is .F.
   llCaseSens = UPPER(GETENV("FTS5")) == "/CS"
   
   lcLogParam = ALLTRIM(UPPER(GETENV("FTS6")))
   
   * Flag indicating redirect output to a file and filename
   llLog = LEFT(lcLogParam, 4) == "/LOG"
   
   IF llLog
      IF LEN(lcLogParam) > 4
         lcLogFile = SUBSTR(lcLogParam, 5)
      ENDIF
   ENDIF
   
ENDIF

RETURN


*!*****************************************************************************
*!
*!       Function: GETWITHIN()
*!
*!      Called by: PARSECMDLN         (procedure in FTS.PRG)
*!
*!*****************************************************************************
FUNCTION GetWithin
PARAMETERS lcString

RETURN lcString


*!*****************************************************************************
*!
*!       Function: BUILDCMD()
*!
*!      Called by: TEXTSEARCH         (procedure in FTS.PRG)
*!
*!*****************************************************************************
FUNCTION BuildCmd
PARAMETERS lcString

IF NOT EMPTY(gcCommand)
   RETURN gcCommand
ENDIF

DO CASE
CASE "#AND#"$UPPER(lcString) OR "#OR#"$UPPER(lcString)
   DO CASE
   CASE "#AND#"$UPPER(lcString) AND NOT "#OR#"$UPPER(lcString)
      
      gcCommand = "llFound = (ATC('"
      gcCommand = gcCommand + LEFT(lcString, AT("#AND#", ;
         lcString) - 1)
      gcCommand = gcCommand + "', lcString) != 0) AND "
      
      lcString = SUBSTR(lcString, AT("#AND#", ;
         lcString) + 5)
      
      gcCommand = gcCommand + "(ATC('" + lcString
      
      gcCommand = gcCommand + "', lcString) != 0)"
      
      DO WHILE "#AND#"$UPPER(lcString)
         
      ENDDO
      
   CASE "#OR#"$UPPER(lcString) AND NOT "#AND#"$UPPER(lcString)
      
   ENDCASE
   
CASE OCCURS("#NOT#", UPPER(lcString)) = 1
   lcString1 = LEFT(lcString, AT("#NOT#", lcString) - 1)
   lcString2 = SUBSTR(lcString, AT("#NOT#", lcString) + 5)
   
   gcCommand = "llFound = (ATC(lcString1, lcString) != 0)" + ;
      " AND (ATC(lcString2, lcString) = 0)"
   
OTHERWISE
   gcCommand = "llFound = (ATC(lcSrchText, lcString) != 0)"
   
ENDCASE

* If case sensitive, use AT(), otherwise use ATC().
IF llCaseSens
   RETURN STRTRAN(gcCommand, "ATC(", "AT(")
ELSE
   RETURN gcCommand
ENDIF


******************************************************************************
*
*      Procedure:  EscHandler
*
*
******************************************************************************
PROCEDURE EscHandler
CLEAR
WAIT CLEAR
WAIT WINDOW "Escape pressed... Press <Enter> to cancel." TIMEOUT 11
DO CleanUp
CANCEL

*: EOF: FTS.PRG
